#![allow(non_snake_case)]

extern crate MBrane;

use MBrane::Vector::Vector2D::*;

#[test]
fn vector2d_initialization() {
    let vector = Vector2D{X: 65.0, Y: 72.5};

    assert_eq!(65.0, vector.X);
    assert_eq!(72.5, vector.Y);
}

#[test]
fn vector2d_magnitude() {
    let vector = Vector2D{X: 12.0, Y: 18.0};
    
    let result: f64 = vector.magnitude();

    assert_eq!(21.633307652783937, result);
}

#[test]
fn vector2d_normalization() {
    let mut vector = Vector2D{X: 12.0, Y: 18.0};
    
    vector.normalize();
    
    assert_eq!(0.554700196225229, vector.X);
    assert_eq!(0.8320502943378436, vector.Y);
}

#[test]
fn vector2d_addition() {
    let lhs = Vector2D{X: 65.0, Y: 10.0};
    let rhs = Vector2D{X: 65.0, Y: 10.0};

    let result = lhs  + rhs;

    assert_eq!(130.0, result.X);
    assert_eq!(20.0, result.Y);
}

#[test]
fn vector2d_substraction() {
    let lhs = Vector2D{X: 12.0, Y: 25.0};
    let rhs = Vector2D{X: 6.0, Y: 10.0};

    let result = lhs - rhs;

    assert_eq!(6.0, result.X);
    assert_eq!(15.0, result.Y);
}

#[test]
fn vector2d_multiplication() {
    let lhs = Vector2D{X: 5.0, Y: 12.0};
    let rhs = Vector2D{X: 5.0, Y: 7.0};

    let result = lhs * rhs;

    assert_eq!(25.0, result.X);
    assert_eq!(84.0, result.Y);
}

#[test]
fn vector2d_division() {
    let lhs = Vector2D{X: 100.0, Y: 50.0};
    let rhs = Vector2D{X: 2.0, Y: 2.0};

    let result = lhs / rhs;

    assert_eq!(50.0, result.X);
    assert_eq!(25.0, result.Y);
}
