#![allow(non_snake_case)]

extern crate MBrane;

use MBrane::Vector::Vector3D::*;

#[test]
fn vector3d_initialization() {
    let vector = Vector3D{X: 65.0, Y: 72.5, Z: 1.5};
    
    assert_eq!(65.0, vector.X);
    assert_eq!(72.5, vector.Y);
    assert_eq!(1.5, vector.Z);
}

#[test]
fn vector3d_magnitude() {
    let vector = Vector3D{X: 4.0, Y: 3.0, Z: 2.0};
    
    assert_eq!(5.385164807134504, vector.magnitude());
}

#[test]
fn vector3d_normalization() {
    let mut vector = Vector3D{X: 4.0, Y: 3.0, Z: 2.0};
    
    vector.normalize();
    
    assert_eq!(0.7427813527082074, vector.X);
    assert_eq!(0.5570860145311556, vector.Y);
    assert_eq!(0.3713906763541037, vector.Z);
}

#[test]
fn vector3d_addition() {
    let lhs = Vector3D{X: 65.0, Y: 10.0, Z: 5.50};
    let rhs = Vector3D{X: 65.0, Y: 10.0, Z: 5.50};

    let result = lhs  + rhs;

    assert_eq!(130.0, result.X);
    assert_eq!(20.0, result.Y);
    assert_eq!(11.00, result.Z);
}

#[test]
fn vector3d_substraction() {
    let lhs = Vector3D{X: 12.0, Y: 25.0, Z: 10.5};
    let rhs = Vector3D{X: 6.0, Y: 10.0, Z: 5.5};

    let result = lhs - rhs;

    assert_eq!(6.0, result.X);
    assert_eq!(15.0, result.Y);
    assert_eq!(5.0, result.Z);
}

#[test]
fn vector3d_multiplication() {
    let lhs = Vector3D{X: 5.0, Y: 12.0, Z: 2.0};
    let rhs = Vector3D{X: 5.0, Y: 7.0, Z: 60.0};

    let result = lhs * rhs;

    assert_eq!(25.0, result.X);
    assert_eq!(84.0, result.Y);
    assert_eq!(120.0, result.Z);
}

#[test]
fn vector3d_division() {
    let lhs = Vector3D{X: 100.0, Y: 50.0, Z: 360.0};
    let rhs = Vector3D{X: 2.0, Y: 2.0, Z: 2.0};

    let result = lhs / rhs;

    assert_eq!(50.0, result.X);
    assert_eq!(25.0, result.Y);
    assert_eq!(180.0, result.Z);
}
